@@ -57,7 +57,7 @@ public:
         return converter_type::get( fun( lhs[i], rhs[j] ) );
     }
 
-    inline int size() const { return nr * nc ; }
+    inline R_xlen_t size() const { return static_cast<R_xlen_t>(nr) * nc ; }
     inline int nrow() const { return nr; }
     inline int ncol() const { return nc; }
 
